using CookComputing.XmlRpc;
using System.IO;

//We use the XMLRPC library from
// xml-rpc.net


class UniversignRequests
{
    private string email;
    private string password;

    public UniversignRequests(string email, string password)
    {
        this.email = email;
        this.password = password;
    }

    private ISignature init()
    {
        //Initialize the proxy
        ISignature proxy = (ISignature)XmlRpcProxyGen.Create(typeof(ISignature));
        proxy.Url = "https://ws.universign.eu/sign/rpc";
        //set credentials
        proxy.Credentials = new System.Net.NetworkCredential(email, password);
        //The library fails if set to True
        proxy.KeepAlive = false;

        return proxy;
    }

    public void signatureDocument(string document)
    {
        ISignature proxy = init();
        
        SignatureField sfield = new SignatureField();
        sfield.page = 1;
        sfield.x = 30;
        sfield.y = 30;

        // Structure TransactionSigner
        TransactionSigner tsign = new TransactionSigner();
        tsign.firstname = "Martin";
        tsign.lastname = "Dupont";
        tsign.emailAddress = "email@sample.com";
        //required only if sms validation used 
        //tsign.phoneNum = "33600000000";
        tsign.signatureField = sfield;

        // Structure TransactionDocument
        TransactionDocument tdoc = new TransactionDocument();
        tdoc.name = Path.GetFileName(document);
        tdoc.content = File.ReadAllBytes(document);

        // Structure TransactionRequest
        TransactionRequest treq = new TransactionRequest();
        //replace by your application URL
        //The loading of this page should trigger the retrieveDocument call
        treq.successURL = "https://www.universign.eu/fr/landing/";
        treq.failURL = "https://www.universign.eu/fr/landing/?res=fail";
        treq.cancelURL = "https://www.universign.eu/fr/landing/?res=cancel";
        treq.profile = "default";
        treq.signers = new TransactionSigner[1];
        treq.signers[0] = tsign;
        treq.documents = new TransactionDocument[1];
        treq.documents[0] = tdoc;
        treq.mustContactFirstSigner = false;
        treq.finalDocSent = false;
        treq.identificationType = "none";
        treq.certificateTypes = new string[1];
        treq.certificateTypes[0] = "timestamp";
        treq.language = "fr";
        treq.handwrittenSignature = false;

        try {
            //request a new transaction
            TransactionResponse trep = proxy.requestTransaction(treq);

            //print the ID: used to retrieve the document after the signer signature
            System.Console.WriteLine("ID: " + trep.id);
            //print the URL: the user should be redirected to this URL
            System.Console.WriteLine("URL: " + trep.url);
        } catch (System.Exception ex) {
            System.Console.WriteLine(ex.ToString());
        }
    }

    public void retrieveDocument(string id, string outPrefix)
    {
        ISignature proxy = init();

        try {
            //retrieve the documents
            TransactionDocument[] docs = proxy.getDocuments(id);

            //in the general case: more than one PDFs
            //however docs[0] is ALWAYS safe if there is only one PDF
            for (int i = 0; i < docs.Length; i++) {
                File.WriteAllBytes(outPrefix + "-" + i + ".pdf", docs[i].content);
            }
        } catch (System.Exception ex) {
            System.Console.WriteLine(ex.ToString());
        }
    }

    public TransactionInfo getInfo(string id)
    {
        ISignature proxy = init();
        
        try {
            return proxy.getTransactionInfo(id);
        } catch (System.Exception ex) {
            System.Console.WriteLine(ex.ToString());
            throw ex;
        }
    }

    static void Main(string[] args)
    {
        UniversignRequests reqs = new UniversignRequests("email", "password");
        reqs.signatureDocument("contract.pdf");
        //reqs.retrieveDocument("00000000-0000-0000-0000-00000000", "doc");
        //TransactionInfo info = reqs.getInfo("00000000-0000-0000-0000-00000000");
    }
}
